// Copyright (C) 2026 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-3.0-only

#ifndef QTQUICK3DAUDIOGLOBAL_H
#define QTQUICK3DAUDIOGLOBAL_H

#include <QtSpatialAudio/qtspatialaudioglobal.h>
#include <QtQuick3DSpatialAudio/qtquick3dspatialaudioexports.h>

#endif // QTQUICK3DAUDIOGLOBAL_H
