#ifndef QT_FEATURES_Multimedia_src_multimedia_qtmultimedia_config_h_H
#define QT_FEATURES_Multimedia_src_multimedia_qtmultimedia_config_h_H

#define QT_FEATURE_alsa -1

#define QT_FEATURE_avfoundation -1

#define QT_FEATURE_coreaudio -1

#define QT_FEATURE_videotoolbox -1

#define QT_FEATURE_gstreamer_qt_api -1

#define QT_FEATURE_mmrenderer -1

#define QT_FEATURE_native_android_backend -1

#define QT_FEATURE_native_windows_backend -1

#define QT_FEATURE_native_darwin_backend -1

#define QT_FEATURE_native_ohos_backend -1

#define QT_FEATURE_pulseaudio 1


#endif // QT_FEATURES_Multimedia_src_multimedia_qtmultimedia_config_h_H
